@@ -342,6 +342,8 @@ typedef struct slurm_msg {
 			 * our trust store) */
 	void *conn; /* interfaces/conn data used for sending this message and
 		     * receiving a response */
+	bool conn_is_mtls; /* True if the connection used to receive this message
+			    * used mTLS */
 
 	uint16_t msg_type; /* really a slurm_msg_type_t but needs to be
 			    * this way for packing purposes.  message type */
