@@ -66,6 +66,7 @@
 #define DEFAULT_SYSTEM_CGSLICE "system.slice"
 #define SYSTEM_CGSCOPE "slurmstepd"
 #define SYSTEM_CGDIR "system"
+#define SLURMD_CGROUP "slurmd"
 
 const char plugin_name[] = "Cgroup v2 plugin";
 const char plugin_type[] = "cgroup/v2";
@@ -1273,24 +1274,32 @@ static int _unset_cpu_mem_limits(xcgroup_t *cg)
 }
 
 /*
- * Slurmd started manually may not remain in the actual scope. Normally there
- * are other pids there, like the terminal from where it's been launched, so
- * slurmd would affect these pids. For example a CoreSpecCount of 1 would leave
- * the bash terminal with only one core.
+ * Create a new cgroup, define it as our new root cgroup, set controllers and
+ * move our process into it.
  *
- * Get out of there and put ourselves into a new home. This shouldn't happen on
- * production systems.
+ * Case 1: Slurmd started manually may not remain in the actual scope. Normally
+ * there are other pids there, like the terminal from where it's been launched,
+ * so slurmd would affect these pids. For example a CoreSpecCount of 1 would
+ * leave the bash terminal with only one core.
+ *
+ * Case 2: Slurmd has been launched with systemd. To avoid systemd resetting our
+ * cgroup settings (e.g. cpuset.cpus) on a daemon-reload, move ourselves into a
+ * sub-tree. Simulates "DelegateSubgroup=slurmd" (not available everywhere).
+ *
+ * IN new_path - base path of the new cgroup to set up and move ourselves to
+ * IN name - name of the new cgroup
+ * RET - SLURM_SUCCESS if cgroup setup and migration was successful
  */
-static int _migrate_to_stepd_scope()
+static int _reparent_into_cgroup(char *new_path, char *name)
 {
 	char *new_home = NULL;
 	pid_t slurmd_pid = getpid();
 
+	xstrfmtcat(new_home, "%s/%s", new_path, name);
 	bit_clear_all(int_cg_ns.avail_controllers);
 	xfree(int_cg_ns.mnt_point);
 	common_cgroup_destroy(&int_cg[CG_LEVEL_ROOT]);
 
-	xstrfmtcat(new_home, "%s/slurmd", stepd_scope_path);
 	int_cg_ns.mnt_point = new_home;
 
 	if (common_cgroup_create(&int_cg_ns, &int_cg[CG_LEVEL_ROOT], "",
@@ -1312,18 +1321,6 @@ static int _migrate_to_stepd_scope()
 	 */
 	invoc_id = "";
 
-	if (_get_controllers(stepd_scope_path, int_cg_ns.avail_controllers) !=
-	    SLURM_SUCCESS)
-		return SLURM_ERROR;
-
-	if (_enable_subtree_control(stepd_scope_path,
-				    int_cg_ns.avail_controllers) !=
-	    SLURM_SUCCESS) {
-		error("Cannot enable subtree_control at the top level %s",
-		      int_cg_ns.mnt_point);
-		return SLURM_ERROR;
-	}
-
 	if (common_cgroup_move_process(&int_cg[CG_LEVEL_ROOT], slurmd_pid) !=
 	    SLURM_SUCCESS) {
 		error("Unable to attach slurmd pid %d to %s cgroup.",
@@ -1331,6 +1328,17 @@ static int _migrate_to_stepd_scope()
 		return SLURM_ERROR;
 	}
 
+	if (_get_controllers(new_path, int_cg_ns.avail_controllers) !=
+	    SLURM_SUCCESS)
+		return SLURM_ERROR;
+
+	if (_enable_subtree_control(new_path, int_cg_ns.avail_controllers) !=
+	    SLURM_SUCCESS) {
+		error("Cannot enable subtree_control at the top level %s",
+		      int_cg_ns.mnt_point);
+		return SLURM_ERROR;
+	}
+
 	return SLURM_SUCCESS;
 }
 
@@ -1599,6 +1607,7 @@ end:
 
 extern int cgroup_p_setup_scope(char *scope_path)
 {
+	char *cgroup_root_path;
 	/*
 	 * Detect if we are started by systemd. Another way could be to check
 	 * if our PPID=1, but we cannot rely on it because when starting slurmd
@@ -1701,10 +1710,32 @@ extern int cgroup_p_setup_scope(char *scope_path)
 		 */
 		if (!invoc_id) {
 			log_flag(CGROUP, "assuming slurmd has been started manually.");
-			if (_migrate_to_stepd_scope() != SLURM_SUCCESS)
+			if (_reparent_into_cgroup(stepd_scope_path,
+						  SLURMD_CGROUP) !=
+			    SLURM_SUCCESS)
 				return SLURM_ERROR;
 		} else {
 			log_flag(CGROUP, "INVOCATION_ID env var found. Assuming slurmd has been started by systemd.");
+			/*
+			 * When launching slurmd as a service we need to migrate
+			 * slurmd to be in a subcgroup in its current root. This
+			 * is done because systemd owns the unit root cgroups
+			 * and resets limits on a daemon-reload, even if
+			 * Delegate=yes is explicit. Since systemd 255, this can
+			 * be achieved by setting DelegateSubgroup=slurmd in the
+			 * unit file, so in the future this code might be
+			 * changed.
+			 */
+			cgroup_root_path = xstrdup(int_cg[CG_LEVEL_ROOT].path);
+			if (_reparent_into_cgroup(cgroup_root_path,
+						  SLURMD_CGROUP) !=
+			    SLURM_SUCCESS) {
+				error("Cannot migrate slurmd to %s/%s",
+				      cgroup_root_path, SLURMD_CGROUP);
+				xfree(cgroup_root_path);
+				return SLURM_ERROR;
+			}
+			xfree(cgroup_root_path);
 		}
 
 		/*
@@ -2744,13 +2775,15 @@ extern int cgroup_p_task_addto(cgroup_ctl_type_t ctl, stepd_step_rec_t *step,
 
 extern cgroup_acct_t *cgroup_p_task_get_acct_data(uint32_t task_id)
 {
+	uint64_t active_file, inactive_file;
 	char *cpu_stat = NULL, *memory_stat = NULL, *memory_current = NULL;
 	char *memory_peak = NULL;
 	char *ptr;
 	size_t tmp_sz = 0;
 	cgroup_acct_t *stats = NULL;
 	task_cg_info_t *task_cg_info;
-	static bool interfaces_checked = false, memory_peak_interface = false;
+	bool memory_peak_interface = false, no_file_cache = false;
+	static bool interfaces_checked = false;
 
 	if (!(task_cg_info = list_find_first(task_list, _find_task_cg_info,
 					     &task_id))) {
@@ -2763,6 +2796,9 @@ extern cgroup_acct_t *cgroup_p_task_get_acct_data(uint32_t task_id)
 		return NULL;
 	}
 
+	if (xstrcasestr(slurm_conf.job_acct_gather_params, "no_file_cache"))
+		no_file_cache = true;
+
 	/*
 	 * Check optional interfaces existence and permissions. This check
 	 * will help to avoid querying unexistent cgroup interfaces every time,
@@ -2847,34 +2883,50 @@ extern cgroup_acct_t *cgroup_p_task_get_acct_data(uint32_t task_id)
 		xfree(cpu_stat);
 	}
 
-	/*
-	 * In cgroup/v1, total_rss was the hierarchical sum of # of bytes of
-	 * anonymous and swap cache memory (including transparent huge pages).
-	 *
-	 * In cgroup/v2 we use memory.current which includes all the
-	 * memory the app has touched. Using this value makes it consistent with
-	 * the OOM killer limit.
-	 */
-	if (memory_current) {
-		if (sscanf(memory_current, "%"PRIu64, &stats->total_rss) != 1)
-			error("Cannot parse memory.current file");
-		xfree(memory_current);
-	}
-
 	if (memory_stat) {
 		ptr = xstrstr(memory_stat, "pgmajfault");
 		if (ptr && (sscanf(ptr, "pgmajfault %"PRIu64,
 				   &stats->total_pgmajfault) != 1))
 			log_flag(CGROUP, "Cannot parse pgmajfault field in memory.stat file");
+
+		if (no_file_cache) {
+			ptr = xstrstr(memory_stat, "\nactive_file");
+			if (ptr && (sscanf(ptr, "\nactive_file %" PRIu64,
+					   &active_file) != 1))
+				log_flag(CGROUP, "Cannot parse active_file field in memory.stat file");
+
+			ptr = xstrstr(memory_stat, "\ninactive_file");
+			if (ptr && (sscanf(ptr, "\ninactive_file %" PRIu64,
+					   &inactive_file) != 1))
+				log_flag(CGROUP, "Cannot parse inactive_file field in memory.stat file");
+		}
+
 		xfree(memory_stat);
 	}
 
-	if (memory_peak) {
+	/* memory.current includes all the memory the app has touched. */
+	if (memory_current) {
+		if (sscanf(memory_current, "%"PRIu64, &stats->total_rss) != 1)
+			error("Cannot parse memory.current file");
+
+		if (no_file_cache) {
+			stats->total_rss -= active_file + inactive_file;
+		}
+
+		xfree(memory_current);
+	}
+
+	/*
+	 * memory.peak includes all memory, including filesystem-backed mem, so
+	 * do not provide it if user does not want it.
+	 */
+	if (memory_peak && !no_file_cache) {
 		if (sscanf(memory_peak, "%"PRIu64, &stats->memory_peak) != 1)
 			error("Cannot parse memory.peak file");
-		xfree(memory_peak);
 	}
 
+	xfree(memory_peak);
+
 	return stats;
 }
 
