@@ -135,6 +135,19 @@ typedef struct {
 	uint16_t prev_hres_id;
 } foreach_hres_set_mode3_t;
 
+typedef struct {
+	list_t *licenses_cur;
+	list_t *licenses_next;
+	list_itr_t *licenses_cur_iter;
+	slurmctld_resv_t *job_resv_ptr;
+} find_relevant_hres_diff_args_t;
+
+typedef struct {
+	list_itr_t *licenses_iter;
+	uint16_t hres_id;
+	slurmctld_resv_t *job_resv_ptr;
+} find_diff_hres_in_list_args_t;
+
 static void _print_path(path_idx_t path_idx, uint16_t depth)
 {
 	for (int i = 0; i < depth; i++) {
@@ -601,6 +614,8 @@ static void _add_res_rec_2_lic_list(slurmdb_res_rec_t *rec, bool sync)
 	license_entry->id.lic_id = next_lic_id++;
 	xassert(license_entry->id.lic_id != NO_VAL16);
 
+	license_entry->id.hres_id = NO_VAL16;
+
 	list_append(cluster_license_list, license_entry);
 	last_license_update = time(NULL);
 }
@@ -2147,6 +2162,15 @@ extern list_t *cluster_license_copy(void)
 	return license_list_dest;
 }
 
+extern int cluster_license_count(void)
+{
+	int cnt;
+	slurm_mutex_lock(&license_mutex);
+	cnt = list_count(cluster_license_list);
+	slurm_mutex_unlock(&license_mutex);
+	return cnt;
+}
+
 /*
  * We need to track the allocated licenses separately, so that:
  *
@@ -2179,6 +2203,7 @@ static int _set_licenses_alloc(job_record_t *job_ptr, bool lic_or,
 		xassert(license_entry ==
 			(licenses_t *) list_peek(job_ptr->license_list));
 	}
+	xfree(job_ptr->licenses_allocated);
 	job_ptr->licenses_allocated =
 		license_list_to_string(job_ptr->license_list);
 
@@ -3137,5 +3162,133 @@ static int _bf_licenses_find_difference(void *x, void *key)
 
 extern bool slurm_bf_licenses_equal(bf_licenses_t *a, bf_licenses_t *b)
 {
+	/* The # of licenses can be different based on advanced reservations */
+	if (list_count(a) != list_count(b))
+		return false;
 	return !(list_find_first_ro(a, _bf_licenses_find_difference, b));
 }
+
+static int _find_more_hres_in_list(void *x, void *arg)
+{
+	bf_license_t *lic_next = x;
+	bf_license_t *lic_cur = NULL;
+	find_diff_hres_in_list_args_t *args = arg;
+	bf_licenses_find_resv_t target_record = {
+		.id = lic_next->id,
+		.resv_ptr = lic_next->resv_ptr,
+	};
+
+	if (lic_next->id.hres_id != args->hres_id)
+		return 0;
+
+	if (lic_next->resv_ptr && (args->job_resv_ptr != lic_next->resv_ptr))
+		return 0;
+
+	lic_cur = list_find(args->licenses_iter, _bf_licenses_find_resv,
+			    &target_record);
+
+	/*
+	 * Break out of list_find_first if lic_next has more resources remaining
+	 * the lic_cur.
+	 *
+	 * In context of backfill node_space table licenses, if lic_cur has more
+	 * than or equal resources compared to lic_next then it would be useless
+	 * to try scheduling on licenses_next's timeslot since the job would
+	 * have already failed with licenses_cur's resources. In short don't set
+	 * later_start if there are less or equal resources in the next slot.
+	 */
+
+	/* If lic_cur is NULL then it is like there are zero remaining */
+	if (!lic_cur && !lic_next->remaining)
+		return 0; /* equal remaining since both have none */
+	if (!lic_cur || (lic_next->remaining > lic_cur->remaining))
+		return 1; /* more resources in lic_next - break out */
+
+	return 0;
+}
+
+static int _bf_licenses_find_relevant_hres_increase(void *x, void *key)
+{
+	licenses_t *job_license = x;
+	find_relevant_hres_diff_args_t *args = key;
+	find_diff_hres_in_list_args_t fargs = {
+		.hres_id = job_license->id.hres_id,
+		.job_resv_ptr = args->job_resv_ptr,
+	};
+
+	if (job_license->id.hres_id == NO_VAL16)
+		return 0;
+
+	if (!args->licenses_cur_iter)
+		args->licenses_cur_iter =
+			list_iterator_create(args->licenses_cur);
+	else
+		list_iterator_reset(args->licenses_cur_iter);
+
+	/*
+	 * Iterate through licenses_next and see if any relevant elements
+	 * matching the job's hres request have more hres remaining in
+	 * licenses_next than licenses_cur.
+	 *
+	 * bf_license lists elements are in the same order so pass an iterator
+	 * of licenses_cur so both lists only needed to be iterated through
+	 * once.
+	 */
+	fargs.licenses_iter = args->licenses_cur_iter;
+	if (list_find_first_ro(args->licenses_next, _find_more_hres_in_list,
+			       &fargs))
+		return 1;
+
+	/*
+	 * Even if licenses_cur contains more records, they don't have the same
+	 * hres id or are additional records with a reservation that are no
+	 * longer available in the next time slot.
+	 */
+
+	return 0;
+}
+
+extern bool slurm_bf_licenses_relevant_hres_increase(list_t *current,
+						     list_t *next,
+						     job_record_t *job_ptr)
+{
+	bool next_has_more; /* next has more hres remaining avail to job */
+	find_relevant_hres_diff_args_t args = {
+		.licenses_cur = current,
+		.licenses_next = next,
+		.job_resv_ptr = job_ptr->resv_ptr,
+	};
+
+	if (!job_ptr->license_list)
+		return false;
+
+	next_has_more =
+		list_find_first_ro(job_ptr->license_list,
+				   _bf_licenses_find_relevant_hres_increase,
+				   &args);
+	if (args.licenses_cur_iter)
+		list_iterator_destroy(args.licenses_cur_iter);
+
+	return next_has_more;
+}
+
+/* sort appended resv bf_licenses to be in order of resv id and license id */
+extern int bf_license_cmp(void *x, void *y)
+{
+	bf_license_t *entry_a = *(bf_license_t **) x;
+	bf_license_t *entry_b = *(bf_license_t **) y;
+	int resv_cmp_rc;
+
+	if (!entry_a->resv_ptr && !entry_b->resv_ptr)
+		return 0; /* keep these in cluster_license_list order */
+	if (!entry_a->resv_ptr && entry_b->resv_ptr)
+		return -1;
+	if (entry_a->resv_ptr && !entry_b->resv_ptr)
+		return 1;
+	if ((resv_cmp_rc =
+		     slurm_sort_uint32_list_asc(&entry_a->resv_ptr->resv_id,
+						&entry_b->resv_ptr->resv_id)))
+		return resv_cmp_rc;
+	return slurm_sort_uint16_list_asc(&entry_a->id.lic_id,
+					  &entry_b->id.lic_id);
+}
